@@ -219,7 +219,7 @@ CheckSmoWithPlot <- function(termList)
 {
   #gamlss.Smo.plot.list <- c( "tr", "ga")
   gamlss.Smo.plot.list1 <- c( "tr(", "ga(", "nn(", "pvc(", "mrf(", "mrfa(", "ri(",
-                             "own(" , "re(", "lo(")
+                             "own(" , "re(", "lo(", "pcat(")
   lgamsmol  <- length(gamlss.Smo.plot.list1)
   lsm  <- length(termList)
   res <- rep(0, lsm) 
@@ -547,6 +547,10 @@ whichValueSmo <- CheckSmoList(nmt)
           {
             plot(getSmo(object, what, which=whichValueSmo[i]))
           }
+          if (attr(whichValueSmo, "whichSmo")[i]=="pcat")
+          {
+            plot(getSmo(object, what, which=whichValueSmo[i]))
+          }
           if (attr(whichValueSmo, "whichSmo")[i]=="pvc")
           {
             plot(getSmo(object, what, which=whichValueSmo[i]))
